<?php

/*
Plugin Name: Eurobank WooCommerce Payment Gateway
Plugin URI: https://www.papaki.com
Description: Eurobank Payment Gateway allows you to accept payment through various channels such as Maestro, Mastercard, AMex cards, Diners and Visa cards On your Woocommerce Powered Site.
Version: 1.8.7.1
Author: Papaki
Author URI: https://www.papaki.com
License: GPL-3.0+
License URI: http://www.gnu.org/licenses/gpl-3.0.txt
WC tested up to: 6.2.1
Text Domain: woo-payment-gateway-for-eurobank
Domain Path: /languages
 */

if (!defined('ABSPATH')) {
    exit;
}

require_once plugin_dir_path(__FILE__) . 'encryption.php';
add_action('plugins_loaded', 'woocommerce_eurobank_init', 0);

function woocommerce_eurobank_init()
{

    if (!class_exists('WC_Payment_Gateway')) {
        return;
    }

    load_plugin_textdomain('woo-payment-gateway-for-eurobank', false, dirname(plugin_basename(__FILE__)) . '/languages/');

    /**
     * Gateway class
     */
    class WC_Eurobank_Gateway extends WC_Payment_Gateway
    {

        public function __construct()
        {
            global $woocommerce;

            $this->id = 'eurobank_gateway';
           // $this->icon = apply_filters('eurobank_icon', plugins_url('img/eurobank.svg', __FILE__));
            $this->has_fields = true;
            $this->notify_url = WC()->api_request_url('WC_Eurobank_Gateway');
            $this->method_description = __('Eurobank Payment Gateway allows you to accept payment through various channels such as Maestro, Mastercard, AMex cards, Diners and Visa cards On your Woocommerce Powered Site.', 'woo-payment-gateway-for-eurobank');
            $this->redirect_page_id = $this->get_option('redirect_page_id');
            $this->method_title = __('Credit card via Eurobank', 'woo-payment-gateway-for-eurobank');

            // Load the form fields.
            $this->init_form_fields();

            global $wpdb;

            if ($wpdb->get_var("SHOW TABLES LIKE '" . $wpdb->prefix . "eurobank_transactions'") === $wpdb->prefix . 'eurobank_transactions') {
                // The database table exist
            } else {
                // Table does not exist
                $query = 'CREATE TABLE IF NOT EXISTS ' . $wpdb->prefix . 'eurobank_transactions (id int(11) unsigned NOT NULL AUTO_INCREMENT, merch_ref varchar(50) not null, trans_ticket varchar(32) not null , timestamp datetime default null, PRIMARY KEY (id))';
                $wpdb->query($query);
            }

            // Load the settings.
            $this->init_settings();

            // Define user set variables
            $this->title = sanitize_text_field($this->get_option('title'));
            $this->description = sanitize_text_field($this->get_option('description'));
            $this->eb_PayMerchantId = sanitize_text_field($this->get_option('eb_PayMerchantId'));
            $this->eb_PayMerchantKey = sanitize_text_field($this->get_option('eb_PayMerchantKey'));
            $this->pb_installments = absint($this->get_option('pb_installments'));
            $this->test_mode = sanitize_text_field($this->get_option('test_mode'));
            $this->eb_installments_variation = sanitize_text_field($this->get_option('eb_installments_variation'));
            $this->eb_transactionType = sanitize_text_field($this->get_option('eb_transactionType'));
            $this->eb_render_logo = sanitize_text_field($this->get_option('eb_render_logo'));
            $this->eb_enable_log = sanitize_text_field($this->get_option('eb_enable_log'));
            $this->eb_order_note = sanitize_text_field($this->get_option('eb_order_note'));
            //Actions
            add_action('woocommerce_receipt_eurobank_gateway', array($this, 'receipt_page'));
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));

            // Payment listener/API hook
            add_action('woocommerce_api_wc_eurobank_gateway', array($this, 'check_eurobank_response'));

            if($this->eb_render_logo == "yes") {
                $this->icon = apply_filters('eurobank_icon', plugins_url('img/eurobank.svg', __FILE__));
            }
           // add_action('init','register_session');
        }

        /**
         * Admin Panel Options
         * */
        public function admin_options()
        {
            echo '<h3>' . __('Eurobank Gateway', 'woo-payment-gateway-for-eurobank') . '</h3>';
            echo '<p>' . __('Eurobank Gateway allows you to accept payment through various channels such as Maestro, Mastercard, AMex cards, Diners  and Visa cards.', 'woo-payment-gateway-for-eurobank') . '</p>';

            echo '<table class="form-table">';
            $this->generate_settings_html();
            echo '</table>';
        }
        function register_session(){
            if( !session_id() )
                session_start();
        }
        /**
         * Initialise Gateway Settings Form Fields
         * */
        function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'checkbox',
                    'label' => __('Enable Eurobank Gateway', 'woo-payment-gateway-for-eurobank'),
                    'description' => __('Enable or disable the gateway.', 'woo-payment-gateway-for-eurobank'),
                    'desc_tip' => true,
                    'default' => 'yes',
                ),
                'test_mode' => array(
                    'title' => __('Test Environment', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'checkbox',
                    'label' => __('Enable Eurobank Test Environment', 'woo-payment-gateway-for-eurobank'),
                    'description' => __('Enable or disable the test environment.', 'woo-payment-gateway-for-eurobank'),
                    'desc_tip' => true,
                    'default' => 'no',
                ),
                'title' => array('
                    title' => __('Title', 'woo-payment-gateway-for-eurobank'), 
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'woo-payment-gateway-for-eurobank'), 'desc_tip' => false,
                    'default' => __('Credit card via Eurobank', 'woo-payment-gateway-for-eurobank')
                ), 
                'description' => array(
                    'title' => __('Description', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'textarea', 
                    'description' => __('This controls the description which the user sees during checkout.', 'woo-payment-gateway-for-eurobank'),
                    'default' => __('Pay Via Eurobank: Accepts  Mastercard, Visa cards and etc.', 'woo-payment-gateway-for-eurobank')
                ),
                'eb_render_logo' => array(
                    'title' => __('Display the logo of EuroBank', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'checkbox',
                    'description' => __('Enable to display the logo of EuroBank next to the title which the user sees during checkout.', 'woo-payment-gateway-for-eurobank'),
                    'default' => 'yes'
                ),
                'eb_PayMerchantId' => array(
                    'title' => __('Eurobank Merchant ID', 'woo-payment-gateway-for-eurobank'), 
                    'type' => 'text',
                    'description' => __('Enter Your Eurobank Merchant ID', 'woo-payment-gateway-for-eurobank'), 
                    'default' => '', 
                    'desc_tip' => true
                ),
                'eb_PayMerchantKey' => array(
                    'title' => __('Eurobank Merchant KEY', 'woo-payment-gateway-for-eurobank'), 
                    'type' => 'password',
                    'description' => __('Enter Your Eurobank Merchant KEY', 'woo-payment-gateway-for-eurobank'), 
                    'default' => '', 
                    'desc_tip' => false
                ),
                'redirect_page_id' => array(
                    'title' => __('Return page URL <br />(Successful or Failed Transactions)', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'select', 
                    'options' => $this->pb_get_pages('Select Page'),
                    'description' => __('We recommend you to select the default “Thank You Page”, in order to automatically serve both successful and failed transactions, with the latter also offering the option to try the payment again.<br /> If you select a different page, you will have to handle failed payments yourself by adding custom code.', 'woo-payment-gateway-for-eurobank'),
                    'default' => "-1"
                ),
                'pb_installments' => array(
                    'title' => __('Maximum number of installments regardless of the total order amount', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'select',
                    'options' => $this->pb_get_installments('Select Installments'),
                    'description' => __('1 to 24 Installments,1 for one time payment. You must contact Eurobank first<br /> If you have filled the "Max Number of installments depending on the total order amount"', 'woo-payment-gateway-for-eurobank'),
                ),
                'eb_installments_variation' => array(
                    'title' => __('Maximum number of installments depending on the total order amount', 'woo-payment-gateway-for-eurobank'),
                    'type'  => 'text',
                    'description' => __('Example 80:2, 160:4, 300:8</br> total order greater or equal to 80 -> allow 2 installments, total order greater or equal to 160 -> allow 4 installments, total order greater or equal to 300 -> allow 8 installments</br> Leave the field blank if you do not want to limit the number of installments depending on the amount of the order.', 'woo-payment-gateway-for-eurobank')
                ),
                'eb_transactionType' => array(
                    'title' => __('Pre-Authorize', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'checkbox',
                    'label' => __('Enable to capture preauthorized payments', 'woo-payment-gateway-for-eurobank'),
                    'default' => 'no'
                ),
                'eb_enable_log' => array(
                    'title' => __('Enable Debug mode', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'checkbox',
                    'label' => __('Enabling this will log certain information', 'woo-payment-gateway-for-eurobank'),
                    'default' => 'no',
                    'description' => __('Enabling this (and the debug mode from your wp-config file) will log information, e.g. bank responses, which will help in debugging issues.', 'woo-payment-gateway-for-eurobank')
                ),
                'eb_order_note' => array(
                    'title' => __('Enable 2nd “payment received” email', 'woo-payment-gateway-for-eurobank'),
                    'type' => 'checkbox',
                    'label' => __('Enable sending Customer order note with transaction details', 'woo-payment-gateway-for-eurobank'),
                    'default' => 'no',
                    'description' => __('Enabling this will send an email with the support reference id and transaction id to the customer, after the transaction has been completed (either on success or failure)', 'woo-payment-gateway-for-eurobank')
                )
            );
        }

        public function get_option($key, $empty_value = null)
        {
            $option_value = parent::get_option($key, $empty_value);
            if ($key == 'eb_PayMerchantKey') {
                $decrypted = WC_Payment_Gateway_KeyEncryption::decrypt(base64_decode($option_value), substr(NONCE_KEY, 0, 32));
                $option_value = $decrypted;
            }
            return $option_value;
        }
        function payment_fields()
        {
            global $woocommerce;
            if ($description = $this->get_description()) {
                echo wpautop(wptexturize($description));
            }
            if(absint(get_query_var('order-pay'))) {
                $order_id = absint(get_query_var('order-pay'));
                $order = new WC_Order($order_id);
                $Amount = $order->get_total();
            } else if(!$woocommerce->cart->is_empty()) {
                $Amount = $woocommerce->cart->total;
            }
            $max_installments = $this->pb_installments;
            $installments_variation = $this->eb_installments_variation;

            if(!empty($installments_variation)) {
                $max_installments = 1; // initialize the max installments 
                if(isset($installments_variation) && !empty($installments_variation)) {
                    $installments_split = explode(',', $installments_variation);
                    foreach($installments_split as $key => $value) {
                        $installment = explode(':', $value);
                        if(is_array($installment) && count($installment) != 2) {
                            // not valid rule for installments
                            continue;
                        }
                        if(!is_numeric($installment[0]) || !is_numeric($installment[1])) {
                            // not valid rule for installments                            
                            continue;
                        }
                        if($Amount >= ($installment[0])) {
                            $max_installments = $installment[1];
                        }
                    }
                }
            }

            if ($max_installments > 1) {
                $doseis_field = '<p class="form-row ">
			<label for="' . esc_attr($this->id) . '-card-doseis">' . __('Choose Installments', 'woo-payment-gateway-for-eurobank') . ' <span class="required">*</span></label>
                        <select id="' . esc_attr($this->id) . '-card-doseis" name="' . esc_attr($this->id) . '-card-doseis" class="input-select wc-credit-card-form-card-doseis">
                        ';
                for ($i = 1; $i <= $max_installments; $i++) {
                    $doseis_field .= '<option value="' . $i . '">' . ($i == 1 ? __('Without installments','woo-payment-gateway-for-eurobank') : $i). '</option>';
                }
                $doseis_field .= '</select>
		</p>';
                echo $doseis_field;
            }
        }
        public function validate_eb_PayMerchantKey_field($key, $value)
        {
            $encrypted = WC_Payment_Gateway_KeyEncryption::encrypt($value, substr(NONCE_KEY, 0, 32));
            return base64_encode($encrypted);
        }
        function pb_get_installments($title = false, $indent = true)
        {

            for ($i = 1; $i <= 24; $i++) {
                $installment_list[$i] = $i;
            }
            return $installment_list;
        }
        function pb_get_pages($title = false, $indent = true)
        {
            $wp_pages = get_pages('sort_column=menu_order');
            $page_list = array();
            if ($title) {
                $page_list[] = $title;
            }

            foreach ($wp_pages as $page) {
                $prefix = '';
                // show indented child pages?
                if ($indent) {
                    $has_parent = $page->post_parent;
                    while ($has_parent) {
                        $prefix .= ' - ';
                        $next_page = get_page($has_parent);
                        $has_parent = $next_page->post_parent;
                    }
                }
                // add to page list array array
                $page_list[$page->ID] = $prefix . $page->post_title;
            }
            $page_list[-1] = __('Thank you page', 'woo-payment-gateway-for-eurobank');
            return $page_list;
        }

        function calculate_digest($input)
        {  
            $digest = base64_encode(hash('sha256', ($input), true));
            return $digest;
        }

        /**
         * Generate the  Eurobank Payment button link
         * */
        function generate_eurobank_form($order_id)
        {
            global $woocommerce;
            global $wpdb;
            // $englishLocale = array('en_US', 'en_AU', 'en_CA', 'en_GB', 'en_NZ', 'en_ZA');
            $availableLocales = array (
                'en' => 'en',
                'en_US' => 'en',
                'en_AU' => 'en',
                'en_CA' => 'en',
                'en_GB' => 'en',
                'en_NZ' => 'en',
                'en_ZA' => 'en',
                'el' => 'el'
            );
            // $currency = get_woocommerce_currency();
            $version = 2;
            $currency = 'EUR';
            $locale = get_locale();
            if(isset($availableLocales[$locale])) {
                $lang = $availableLocales[$locale];
             } else {
                 $lang = 'en';
             }
            
            $order = new WC_Order($order_id);
            //echo $this->pb_authorize;

            $installments = 1;
            /* if ($this->pb_installments > 1) {
            $installments = intval($order->get_total() / 30);
            $installments = min($installments, $this->pb_installments);
            } */
            
            if (method_exists($order, 'get_meta')) {
                $installments = $order->get_meta('_doseis');
                if ($installments == '') {
                    $installments = 1;
                }
            } else {
                $installments = get_post_meta($order_id, '_doseis', 1);
            }

            if($this->eb_transactionType == 'yes') {
                $trType = 2;
            } else {
                $trType = 1;
            }

            //  store TranTicket in table
            $wpdb->delete($wpdb->prefix . 'eurobank_transactions', array('merch_ref' => $order_id));
            $wpdb->insert($wpdb->prefix . 'eurobank_transactions', array('trans_ticket' => $order_id, 'merch_ref' => $order_id, 'timestamp' => current_time('mysql', 1)));

            //redirect to payment
            $countries_obj = new WC_Countries();
            $country = sanitize_text_field($order->get_billing_country());
            $country_states_array = $countries_obj->get_states();
            $state_code = sanitize_text_field($order->get_billing_state());
            $state = $country_states_array[$country][$state_code];

            wc_enqueue_js('
             $.blockUI({
             message: "' . esc_js(__('Thank you for your order. We are now redirecting you to Eurobank to make payment.', 'woo-payment-gateway-for-eurobank')) . '",
             baseZ: 99999,
             overlayCSS:
             {
             background: "#fff",
             opacity: 0.6
             },
             css: {
             padding:        "20px",
             zindex:         "9999999",
             textAlign:      "center",
             color:          "#555",
             border:         "3px solid #aaa",
             backgroundColor:"#fff",
             cursor:         "wait",
             lineHeight:		"24px",
             }
             });
             jQuery("#eb_payment_form").submit();
             ');

            $billing = sanitize_text_field($order->get_address());
            $_SESSION['order_id'] = $order_id;
            WC()->session->set('eb_order_id', $order_id);


            if($country != 'GR' ) {
                if ($installments > 1) {
                    $form_data_array = array(
                        'version' => $version,
                        'mid' => esc_attr($this->eb_PayMerchantId), 
                        'lang' => $lang, 'deviceCategory' => '0', 
                        'orderid' => $order_id.'at'.date('Ymdhisu'),
                        'orderDesc' => 'Order #' . $order_id,
                        'orderAmount' => $order->get_total(), 
                        'currency' => $currency, 
                        'payerEmail' => $order->get_billing_email(),
                        'billCountry' => $country,
                        'billState' => $state_code,
                        'billZip' => $order->get_billing_postcode(),
                        'billCity' => $order->get_billing_city(),
                        'billAddress' => $order->get_billing_address_1(),
                        'trType' => $trType,
                        'extInstallmentoffset' => 0, 
                        'extInstallmentperiod' => $installments,
                        'confirmUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=success",
                        'cancelUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=failure",
                        'var1' => $order_id
                    );
                } else {
                    $form_data_array = array(
                        'version' => $version,
                        'mid' => esc_attr($this->eb_PayMerchantId), 
                        'lang' => $lang, 
                        'deviceCategory' => '0', 
                        'orderid' => $order_id.'at'.date('Ymdhisu'),
                        'orderDesc' => 'Order #' . $order_id,
                        'orderAmount' => $order->get_total(), 
                        'currency' => $currency, 
                        'payerEmail' => $order->get_billing_email(),
                        'billCountry' => $country,
                        'billState' => $state_code,
                        'billZip' => $order->get_billing_postcode(),
                        'billCity' => $order->get_billing_city(),
                        'billAddress' => $order->get_billing_address_1(),
                        'trType' => $trType,
                        'confirmUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=success",
                        'cancelUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=failure",
                        'var1' => $order_id
                    );
                }
            } else {
                if ($installments > 1) {
                    $form_data_array = array(
                        'version' => $version,
                        'mid' => esc_attr($this->eb_PayMerchantId), 
                        'lang' => $lang, 'deviceCategory' => '0', 
                        'orderid' => $order_id.'at'.date('Ymdhisu'),
                        'orderDesc' => 'Order #' . $order_id,
                        'orderAmount' => $order->get_total(), 
                        'currency' => $currency, 
                        'payerEmail' => $order->get_billing_email(),
                        'billCountry' => $country,
                        'billZip' => $order->get_billing_postcode(),
                        'billCity' => $order->get_billing_city(),
                        'billAddress' => $order->get_billing_address_1(),
                        'trType' => $trType,
                        'extInstallmentoffset' => 0, 
                        'extInstallmentperiod' => $installments,
                        'confirmUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=success",
                        'cancelUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=failure",
                        'var1' => $order_id
                    );
                } else {
                    $form_data_array = array(
                        'version' => $version,
                        'mid' => esc_attr($this->eb_PayMerchantId), 
                        'lang' => $lang, 
                        'deviceCategory' => '0', 
                        'orderid' => $order_id.'at'.date('Ymdhisu'),
                        'orderDesc' => 'Order #' . $order_id,
                        'orderAmount' => $order->get_total(), 
                        'currency' => $currency, 
                        'payerEmail' => $order->get_billing_email(),
                        'billCountry' => $country,
                        'billZip' => $order->get_billing_postcode(),
                        'billCity' => $order->get_billing_city(),
                        'billAddress' => $order->get_billing_address_1(),
                        'trType' => $trType,
                        'confirmUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=success",
                        'cancelUrl' => get_site_url() . "/?wc-api=WC_Eurobank_Gateway&result=failure",
                        'var1' => $order_id
                    );
                }
            }

            $form_secret = $this->eb_PayMerchantKey;
            $form_data = iconv('utf-8', 'utf-8//IGNORE', implode("", $form_data_array)). $form_secret;
            $digest = $this->calculate_digest($form_data);

            if($this->eb_enable_log == 'yes') {
                error_log( '---- Eurobank Transaction digest -----');
                error_log( 'Data: ' );
                error_log( print_r($form_data, true ) );
                error_log( 'Digest: ' );
                error_log( print_r($digest, true ) );
                error_log( '---- End of Eurobank Transaction digest ----');
            }

            if ($this->test_mode != "yes") {
                $send_it_2 = "https://vpos.eurocommerce.gr/vpos/shophandlermpi";
            } else  {
                $send_it_2 = "https://eurocommerce-test.cardlink.gr/vpos/shophandlermpi";
            } 

            $html = '<form action="' . esc_url($send_it_2) . '" method="post" id="eb_payment_form" target="_top" accept-charset="UTF-8">';
            foreach ($form_data_array as $key => $value) {
                //$html .= '<input type="hidden" id="' . $key . '" name="' . $key . '" value="' . utf8_encode($value) . '"/>';
                $html .= '<input type="hidden" id="' . $key . '" name="' . $key . '" value="' . iconv('utf-8', 'utf-8//IGNORE', $value) . '"/>';
            }
            $html .= '<input type="hidden" id="digest" name="digest" value="' . esc_attr($digest) . '"/>';
            $html .= '</form>';
            return $html;
        }

        /**
         * Process the payment and return the result
         * */
        function process_payment($order_id)
        {

            /*
            get_permalink was used instead of $order->get_checkout_payment_url in redirect in order to have a fixed checkout page to provide to Eurobank
             */

            $order = new WC_Order($order_id);
            $doseis = intval($_POST[esc_attr($this->id) . '-card-doseis']);
            if ($doseis > 0) {
                $this->generic_add_meta($order_id, '_doseis', $doseis);
            }
            $current_version = get_option( 'woocommerce_version', null );
            if (version_compare( $current_version, '2.2.0', '<' )) { //older version
                return array('result' => 'success', 'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay')))));
            } else if (version_compare( $current_version, '2.4.0', '<' )) { //older version
                return array
                    (
                        'result' => 'success',
                        'redirect'	=> add_query_arg('order-pay', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
                    );
            } else if (version_compare( $current_version, '3.0.0', '<' )) { //older version
                return array
                    (
                        'result' => 'success',
                        'redirect'	=> add_query_arg('order-pay', $order->id, add_query_arg('key', $order->order_key, wc_get_page_permalink( 'checkout' )))
                    );
            } else {
                return array('result' => 'success', 
                'redirect' => add_query_arg('order-pay', $order->get_id(), add_query_arg('key', $order->get_order_key(), wc_get_page_permalink('checkout'))));
            }

            return array('result' => 'success', 
            'redirect' => add_query_arg('order-pay', $order->get_id(), add_query_arg('key', $order->get_order_key(), wc_get_page_permalink('checkout'))));
        }

        /**
         * Output for the order received page.
         * */
        function receipt_page($order)
        {
            echo '<p>' . __('Thank you - your order is now pending payment. You should be automatically redirected to Eurobank Paycenter to make payment.', 'woo-payment-gateway-for-eurobank') . '</p>';
            echo $this->generate_eurobank_form($order);
        }

        /**
         * Verify a successful Payment!
         * */
        function check_eurobank_response()
        {

            global $woocommerce;
            global $wpdb;


            if($this->eb_enable_log == 'yes') {
                error_log( '---- Eurobank Response -----');
                error_log( print_r( $_POST, true ) );
                error_log( '---- End of Eurobank Response ----');
            }

            //$post_data_keys=array('mid','orderid','status','orderAmount','digest');
            $mid = filter_var($_POST['mid'], FILTER_SANITIZE_NUMBER_INT);

            $orderid_session = WC()->session->get('eb_order_id');
            $reg = preg_match('/^(.*?)at/', $orderid = filter_var($_POST['orderid'],FILTER_SANITIZE_STRING), $matches);

            if(!empty($matches)) {
                $orderid = $matches[1];
            } else {
                $orderid = $orderid_session;
            }

            if($orderid == '') {
                $orderid = filter_var($_POST['orderid'], FILTER_SANITIZE_STRING);
                error_log("Eurobank: something went wrong with order id " );
                error_log(print_r($_POST, true));
                error_log(print_r($matches, true));
                error_log($orderid_session);
            }

            $orderAmount = filter_var($_POST['orderAmount'], FILTER_SANITIZE_NUMBER_FLOAT);
            $status = filter_var($_POST['status'], FILTER_SANITIZE_STRING);
            $post_digest = filter_var($_POST['digest'], FILTER_SANITIZE_STRING);
            $form_data = '';
            foreach ($_POST as $k => $v) {
                if (!in_array($k, array('_charset_', 'digest', 'submitButton'))) {
                    $form_data .= filter_var($_POST[$k], FILTER_SANITIZE_STRING);
                }
            }

            $form_data .= $this->eb_PayMerchantKey;
            $digest = $this->calculate_digest($form_data);
            if($this->eb_enable_log == 'yes') {
                error_log( '---- Eurobank Response -----');
                error_log( '---- Eurobank Post -----');
                error_log( print_r( $_POST, true ) );
                error_log( '---- Eurobank form data:  -----');
                error_log( print_r( $form_data, true ) );
                error_log( '---- Digest from form data:  -----');
                error_log( print_r( $digest, true ) );
                error_log( '---- End of Eurobank Response ----');
            }
            $order = new WC_Order($orderid);

            if ($digest != $post_digest) {
                $message = __('A technical problem occured. <br />The transaction wasn\'t successful, payment wasn\'t received.', 'woo-payment-gateway-for-eurobank');
                $message_type = 'error';
                $eb_message = array('message' => $message, 'message_type' => $message_type);
                $this->generic_add_meta($orderid, '_eurobank_message', $eb_message);
                //Update the order status
                $order->update_status('failed', 'DIGEST');
                if(version_compare(WOOCOMMERCE_VERSION, '2.5', '<')) {
                    $checkout_url = $woocommerce->cart->get_checkout_url();
                } else {
                    $checkout_url = wc_get_checkout_url();
                }
                wp_redirect($checkout_url);
                exit;
            }
            //AUTHORIZED, CAPTURED,CANCELED,REFUSED,ERROR
            if (isset($_GET['result']) && filter_var($_GET['result'], FILTER_SANITIZE_STRING) == 'success') {

                if ($status == 'ERROR') {
                    $message = __('A technical problem occured. <br />The transaction wasn\'t successful, payment wasn\'t received.', 'woo-payment-gateway-for-eurobank');
                    $message_type = 'error';
                    $eb_message = array('message' => $message, 'message_type' => $message_type);
                    $this->generic_add_meta($orderid, '_eurobank_message', $eb_message);
                    //Update the order status
                    $order->update_status('failed', 'ERROR');
                    if(version_compare(WOOCOMMERCE_VERSION, '2.5', '<')) {
                        $checkout_url = $woocommerce->cart->get_checkout_url();
                    } else {
                        $checkout_url = wc_get_checkout_url();
                    }
                    wp_redirect($checkout_url);
                    exit;
                }

                $paymentRef = filter_var($_POST['paymentRef'], FILTER_SANITIZE_STRING);
                $txId = absint($_POST['txId']);
                if ($status == 'CAPTURED' || $status == 'AUTHORIZED') {
                    $order->payment_complete($paymentRef);  
                      
                    if ($order->get_status() == 'processing') {

                        $order->add_order_note(__('Payment Via Eurobank Bank<br />Transaction ID: ', 'woo-payment-gateway-for-eurobank') . $paymentRef);
                        $message = __('Thank you for shopping with us.<br />Your transaction was successful, payment was received.<br />Your order is currently being processed.', 'woo-payment-gateway-for-eurobank');

                        if($this->eb_order_note == 'yes') {
                            //Add customer order note
                            $order->add_order_note(__('Payment Received.<br />Your order is currently being processed.<br />We will be shipping your order to you soon.<br />Eurobank Bank ID: ', 'woo-payment-gateway-for-eurobank') . $paymentRef, 1);

                        }
                    } else if  ($order->get_status() == 'completed') {
                        $message = __('Thank you for shopping with us.<br />Your transaction was successful, payment was received.<br />Your order is now complete.', 'woo-payment-gateway-for-eurobank');
                        if($this->eb_order_note == 'yes') {
                            //Add customer order note
                            $order->add_order_note(__('Payment Received.<br />Your order is now complete.<br />Eurobank Transaction ID: ', 'woo-payment-gateway-for-eurobank') . $paymentRef, 1);
                        }
                    }     
                    $message_type = 'success';

                    $eb_message = array('message' => $message, 'message_type' => $message_type);

                    $this->generic_add_meta($orderid, '_eurobank_message', $eb_message);

                    // Empty cart 
                   WC()->cart->empty_cart();
         
                } else if ($status == 'CANCELED') {
                    if(version_compare(WOOCOMMERCE_VERSION, '2.5', '<')) {
                        $checkout_url = $woocommerce->cart->get_checkout_url();
                    } else {
                        $checkout_url = wc_get_checkout_url();
                    }
                    $message = __('Thank you for shopping with us. <br />However, the transaction wasn\'t successful, payment was cancelled.', 'woo-payment-gateway-for-eurobank');
                    $message_type = 'notice';
                //    wc_add_notice($message, $message_type);
                    //wp_redirect($checkout_url);
                    //exit ;
                } else { //Failed Response codes
                    $message = __('Thank you for shopping with us. <br />However, the transaction wasn\'t successful, payment wasn\'t received.', 'woo-payment-gateway-for-eurobank');
                    $message_type = 'error';
                    $eb_message = array('message' => $message, 'message_type' => $message_type);
                    $this->generic_add_meta($orderid, '_eurobank_message', $eb_message);
                    //wc_add_notice( $message, $message_type );
                    $order->update_status('failed', 'UNKNOWN');
                }
            }
            if (isset($_GET['result']) && (filter_var($_GET['result'], FILTER_SANITIZE_STRING) == 'failure')) {

                $order = new WC_Order($orderid);
                $message = __('Thank you for shopping with us. <br />However, the transaction wasn\'t successful, payment wasn\'t received.', 'woo-payment-gateway-for-eurobank');
                $message_type = 'error';
                // wc_add_notice( $message, $message_type );
                $transaction_id = absint($_POST['txId']);

                if($this->eb_order_note == 'yes') {
                    //Add Customer Order Note
                    $order->add_order_note($message . '<br />Eurobank Transaction ID: ' . $transaction_id, 1);
                }

                //Add Admin Order Note
                $order->add_order_note($message . '<br />Eurobank Transaction ID: ' . $transaction_id . '');

                //Update the order status
                $order->update_status('failed', filter_var($_POST['message'], FILTER_SANITIZE_STRING));

                $eb_message = array('message' => $message, 'message_type' => $message_type);

                $this->generic_add_meta($orderid, '_eurobank_message', $eb_message);
            }

            if ($this->redirect_page_id == "-1") {
                $redirect_url = $this->get_return_url($order);
            } else {
                $redirect_url = ($this->redirect_page_id == "" || $this->redirect_page_id == 0) ? $this->get_return_url($order) : get_permalink($this->redirect_page_id);
            }
            wp_redirect($redirect_url);

            exit;
        }

        function generic_add_meta($orderid, $key, $value)
        {
            $order = new WC_Order($orderid);
            if (method_exists($order, 'add_meta_data') && method_exists($order, 'save_meta_data')) {
                $order->add_meta_data($key, $value, true);
                $order->save_meta_data();
            } else {
                update_post_meta($orderid, $key, $value);
            }
        }

    }

    function eurobank_message()
    {
        $order_id = absint(get_query_var('order-received'));
        $order = new WC_Order($order_id);
        if (method_exists($order, 'get_payment_method')) {
            $payment_method = $order->get_payment_method();
        } else {
            $payment_method = $order->payment_method;
        }
        if (is_order_received_page() && ('eurobank_gateway' == $payment_method)) {
            if (method_exists($order, 'get_meta')) {
                $eurobank_message = $order->get_meta('_eurobank_message', true);
            } else {
                $eurobank_message = get_post_meta($order_id, '_eurobank_message');
            }

            if (!empty($eurobank_message)) {
                $message = $eurobank_message['message'];
                $message_type = $eurobank_message['message_type'];
                if (method_exists($order, 'delete_meta_data')) {
                    $order->delete_meta_data('_eurobank_message');
                    $order->save_meta_data();
                } else {
                    delete_post_meta($order_id, '_eurobank_message');
                }
                wc_add_notice($message, $message_type);
            }
        }
    }

    add_action('wp', 'eurobank_message');

    /**
     * Add Eurobank Gateway to WC
     * */
    function woocommerce_add_eurobank_gateway($methods)
    {
        $methods[] = 'WC_Eurobank_Gateway';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'woocommerce_add_eurobank_gateway');

    /**
     * Add Settings link to the plugin entry in the plugins menu for WC below 2.1 
     * */
    if (version_compare(WOOCOMMERCE_VERSION, "2.1") <= 0) {

        add_filter('plugin_action_links', 'eurobank_plugin_action_links', 10, 2);

        function eurobank_plugin_action_links($links, $file)
        {
            static $this_plugin;

            if (!$this_plugin) {
                $this_plugin = plugin_basename(__FILE__);
            }

            if ($file == $this_plugin) {
                $settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=woocommerce_settings&tab=payment_gateways&section=WC_eurobank_Gateway">Settings</a>';
                array_unshift($links, $settings_link);
            }
            return $links;
        }

    }
    /**
     * Add Settings link to the plugin entry in the plugins menu for WC 2.1 and above
     * */else {
        add_filter('plugin_action_links', 'eurobank_plugin_action_links', 10, 2);

        function eurobank_plugin_action_links($links, $file)
        {
            static $this_plugin;

            if (!$this_plugin) {
                $this_plugin = plugin_basename(__FILE__);
            }

            if ($file == $this_plugin) {
                $settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wc-settings&tab=checkout&section=WC_Eurobank_Gateway">Settings</a>';
                array_unshift($links, $settings_link);
            }
            return $links;
        }

    }
}
